import 'package:flutter/material.dart';
import 'package:yin_ui/theme/index.dart';
import 'package:yin_ui/utils.dart';
import 'package:yin_ui/actionsheet/utils.dart';

class IosWidget extends StatefulWidget {
  //标题
  final dynamic title;

  //蒙版可否点击
  final bool maskClosable;

  //取消按钮
  final dynamic cancelButton;

  //关闭
  final Function() close;

  //变化
  final Function(int index) onChange;

  //子视图
  final List<dynamic> options;

  IosWidget({
    key,
    this.title,
    this.maskClosable,
    this.cancelButton,
    this.close,
    this.onChange,
    this.options,
  }) : super(key: key);

  @override
  IosWidgetState createState() => new IosWidgetState();
}

class IosWidgetState extends State<IosWidget> with SingleTickerProviderStateMixin {
  final GlobalKey _boxKey = GlobalKey();
  double _boxHeight = 0;
  int _index;
  AnimationController _controller;
  Animation<double> top;
  Animation<double> opacity;
  WeTheme theme;

  @override
  void initState() {
    super.initState();
    _controller = AnimationController(duration: const Duration(milliseconds: 150), vsync: this);
    WidgetsBinding.instance.addPostFrameCallback(getBoxHeight);
  }

  // 渲染title
  Widget renderTitle() {
    return Container(
      height: 50,
      alignment: Alignment.center,
      decoration: BoxDecoration(
        border: Border(bottom: BorderSide(color: theme.defaultBorderColor, width: 1)),
      ),
      child: DefaultTextStyle(
        style: TextStyle(fontSize: 16, color: Colors.black87, fontWeight: FontWeight.w500),
        child: toTextWidget(widget.title, 'title'),
      ),
    );
  }

  // 渲染内容视图
  Widget renderContent() {
    return ConstrainedBox(
      constraints: BoxConstraints(maxHeight: MediaQuery.of(context).size.height * 0.6),
      child: ListView(
        children: initChilder(widget.options, itemClick, theme.defaultBorderColor),
        padding:EdgeInsets.all(0),
        shrinkWrap: true,
      ),
    );
  }

  // 取消按钮
  Widget renderCancelButton() {
    return InkWell(
      onTap: close,
      child: DecoratedBox(
        decoration: BoxDecoration(
            border: Border(top: BorderSide(width: 6.0, color: Color(0xffEFEFF4))),
            color: Colors.white
        ),
        child: SizedBox(
          height: 55.0,
          child: Align(
            alignment: Alignment.center,
            child: DefaultTextStyle(
              style: TextStyle(fontSize: 16.0, color: Colors.black),
              child: toTextWidget(widget.cancelButton, 'cancelButton'),
            ),
          ),
        ),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    final List<Widget> childs = [];

    //判断标题
    if (widget.title != null) {
      childs.add(renderTitle());
    }

    // 选项
    childs.add(renderContent());

    // 取消按钮
    if (widget.cancelButton != null) {
      childs.add(renderCancelButton());
    }

    return AnimatedBuilder(
      animation: _controller,
      builder: (context, child) {
        return Stack(
          children: <Widget>[
            Positioned(
              top: 0,
              bottom: 0,
              left: 0,
              right: 0,
              child: GestureDetector(
                onTap: () {
                  if (widget.maskClosable) close();
                },
                child: Opacity(
                  opacity: opacity == null ? 0.0 : opacity.value,
                  child: SizedBox(
                    child: DecoratedBox(decoration: BoxDecoration(color: theme.maskColor)),
                  ),
                ),
              ),
            ),
            Positioned(
              left: 0,
              right: 0,
              bottom: 0,
              child: Transform.translate(
                offset: Offset(0, top == null ? 10000 : top.value),
                child: DecoratedBox(
                  key: _boxKey,
                  decoration: BoxDecoration(color: Colors.white),
                  child: Material(
                    child: Column(
                      mainAxisSize: MainAxisSize.min,
                      crossAxisAlignment: CrossAxisAlignment.stretch,
                      children: childs,
                    ),
                  ),
                ),
              ),
            ),
          ],
        );
      },
    );
  }

  // 获取容器高度
  void getBoxHeight(Duration time) {
    _boxHeight = _boxKey.currentContext.size.width;
    createAnimate();
    startAnimate();
  }

  // 创建动画
  void createAnimate() {
    // 内容动画
    top = Tween<double>(begin: _boxHeight, end: 0)
        .animate(CurvedAnimation(parent: _controller, curve: Curves.decelerate))
          ..addStatusListener(animateEnd);

    // 遮罩层透明动画
    opacity =
        Tween<double>(begin: 0.0, end: 1.0).animate(CurvedAnimation(parent: _controller, curve: Curves.decelerate));
  }

  // 开始动画
  void startAnimate() {
    _controller.forward();
  }

  // 动画结束
  void animateEnd(state) {
    if (state == AnimationStatus.dismissed) {
      if (_index == null) {
        widget.close();
      } else {
        widget.onChange(_index);
      }
      // 销毁
      _controller.dispose();
    }
  }

  // 取消
  void close() {
    _index = null;
    _controller.reverse();
  }

  // item click
  void itemClick(int index) {
    _index = index;
    _controller.reverse();
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    theme = WeUI.getTheme(context);
  }

  @override
  void dispose() {
    if (!_controller.isDismissed) {
      _controller?.dispose();
    }
    super.dispose();
  }
}
