import 'package:flutter/material.dart';
import 'package:yin_ui/animation/fade_in.dart';
import 'package:yin_ui/theme/index.dart';

//方向
enum WeDrawerPlacement { top, right, bottom, left }

//实现方式
enum WeDrawerModel { overlay, dialog }

class DrawerWidget extends StatefulWidget {
  final WeDrawerPlacement placement;
  final Widget child;
  final bool mask;
  final Function() maskClick;
  final int duration;
  final WeDrawerModel model;

  DrawerWidget({
    key,
    this.placement,
    this.child,
    this.mask,
    this.maskClick,
    this.model,
    this.duration = 150,
  }) : super(key: key);

  @override
  DrawerWidgetState createState() => new DrawerWidgetState();
}

class DrawerWidgetState extends State<DrawerWidget> with SingleTickerProviderStateMixin {
  GlobalKey boxKey = GlobalKey();
  AnimationController container;
  Animation<double> offsetAnimation;
  bool isVertical;
  WeTheme theme;

  @override
  void initState() {
    super.initState();

    //判断是否垂直
    isVertical = widget.placement == WeDrawerPlacement.bottom || widget.placement == WeDrawerPlacement.top;

    container = AnimationController(duration: Duration(milliseconds: widget.duration), vsync: this);

    offsetAnimation = Tween<double>(begin: 2000, end: 0).animate(
      CurvedAnimation(
        parent: container,
        curve: Curves.ease,
      ),
    );

    WidgetsBinding.instance.addPostFrameCallback(getBoxHeight);
  }

  //获取计算后的高度
  void getBoxHeight(Duration titme) {
    initAnimation();
  }

  //方向执行动画
  reverseAnimation() async {
    await container.reverse();
  }

  //初始化动画
  void initAnimation() {
    final double size = isVertical ? boxKey.currentContext.size.height : boxKey.currentContext.size.width;

    double begin;

    switch (widget.placement) {
      case WeDrawerPlacement.top:
      case WeDrawerPlacement.left:
        begin = -size;
        break;
      case WeDrawerPlacement.bottom:
      case WeDrawerPlacement.right:
        begin = size;
        break;
    }

    offsetAnimation = Tween<double>(begin: begin, end: 0).animate(
      CurvedAnimation(
        parent: container,
        curve: Curves.ease,
      ),
    );
    //执行动画
    container.forward();
  }

  @override
  Widget build(BuildContext context) {
    Offset offset;
    double top = 0.0;
    double right = 0.0;
    double bottom = 0.0;
    double left = 0.0;

    //判断方向
    switch (widget.placement) {
      case WeDrawerPlacement.top:
        bottom = null;
        break;
      case WeDrawerPlacement.left:
        right = null;
        break;
      case WeDrawerPlacement.bottom:
        top = null;
        break;
      case WeDrawerPlacement.right:
        left = null;
        break;
    }

    return Scaffold(
      backgroundColor: Colors.transparent,
      body: AnimatedBuilder(
        animation: container,
        builder: (context, child) {
          if (isVertical) {
            offset = Offset(0, offsetAnimation.value);
          } else {
            offset = Offset(offsetAnimation.value, 0);
          }

          final List<Widget> children = [
            Positioned(
              top: top,
              bottom: bottom,
              right: right,
              left: left,
              child: Transform.translate(
                offset: offset,
                child: DecoratedBox(
                  key: boxKey,
                  decoration: BoxDecoration(color: Colors.white),
                  child: widget.child,
                ),
              ),
            ),
          ];

          //判断是否添加遮层
          if (widget.mask) {
            children.insert(
              0,
              Positioned(
                top: 0,
                bottom: 0,
                left: 0,
                right: 0,
                child: FadeIn(
                  autPlay: false,
                  controller: container,
                  child: GestureDetector(
                    onTap: widget.maskClick,
                    child: DecoratedBox(
                      decoration: BoxDecoration(
                        color: widget.model == WeDrawerModel.dialog ? Colors.transparent : theme.maskColor,
                      ),
                    ),
                  ),
                ),
              ),
            );
          }
          return Stack(children: children);
        },
      ),
    );
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    theme = WeUI.getTheme(context);
  }
}
