import 'package:flutter/material.dart';
import 'package:yin_ui/drawer/drawer.dart';
import 'package:yin_ui/drawer/index.dart';
import 'package:yin_ui/theme/index.dart';
import 'package:yin_ui/utils.dart';

enum PickerType {
  first,
  now,
  last,
}

///底部显示时间选择
void showBottomPicker(
  BuildContext context, {
  PickerType type = PickerType.now,
  dynamic cancelButton = '取消',
  dynamic confirmButton = '确定',
  String title = '请选择',
  List<List<WePickerItem>> options,
  Map<int, int> defaultValue,
  Function(List<WePickerItem>) onConfirm,
}) async {
  DateTime now = DateTime.now();

  List<WePickerItem> years = [];
  List<WePickerItem> mouths = [];
  List<WePickerItem> days = [];
  if (options == null || options.length == 0) {
    for (var i = -99; i < 100; ++i) {
      years.add(WePickerItem(label: '${i + now.year}', value: '${i + now.year}'));
    }

    mouths = List.generate(12, (index) {
      return WePickerItem(label: '${index + 1}', value: '${index + 1}');
    }).toList();

    days = List.generate(31, (index) {
      return WePickerItem(label: '${index + 1}', value: '${index + 1}');
    }).toList();
  } else {
    years = options[0];
    mouths = options[1];
    days = options[2];
  }

  Function closeDrawer;
  Map<int, int> tempValue;
  if (type == PickerType.first) {
    tempValue = {0: 0, 1: 0, 2: 0};
  } else if (type == PickerType.now) {
    tempValue = {0: years.length ~/ 2, 1: now.month - 1, 2: now.day - 1};
  } else {
    tempValue = {0: years.length - 1, 1: mouths.length - 1, 2: days.length - 1};
  }

  closeDrawer = await weDrawer(context)(
    placement: WeDrawerPlacement.bottom,
    child: WePickerView(
      defaultValue: defaultValue ?? tempValue,
      onCancel: () => closeDrawer(),
      onConfirm: (value) {
        closeDrawer();
        if (onConfirm is Function) onConfirm(value);
      },
      cancelButton: cancelButton,
      confirmButton: confirmButton,
      title: title,
      options: [years, mouths, days],
    ),
  );
}

typedef void PickerCallBack(List<WePickerItem> value);

class WePickerView extends StatefulWidget {
  //item 数量
  final int itemCount;

  //item 高度
  final double itemHeight;

  // 滚动回调
  final PickerCallBack onChange;

  // 显示item数据
  final List<List<WePickerItem>> options;

  // 默认显示位置
  final Map<int, int> defaultValue;

  //取消按钮
  final dynamic cancelButton;

  //确定按钮
  final dynamic confirmButton;

  //标题
  final dynamic title;

  //取消回调
  final Function onCancel;

  //确定回调
  final Function onConfirm;

  WePickerView({
    itemCount = 5,
    this.itemHeight = 42,
    this.onChange,
    @required this.options,
    this.defaultValue,
    this.cancelButton,
    this.confirmButton,
    this.title,
    this.onCancel,
    this.onConfirm,
  })  : assert(itemCount % 2 == 1),
        this.itemCount = itemCount;

  @override
  WePickerViewState createState() => WePickerViewState();
}

class WePickerViewState extends State<WePickerView> {
  final Map<int, double> tops = {};
  double boxHeight;
  double borderTop;
  double _startY;
  double _endY = 0;
  WeTheme theme;

  @override
  initState() {
    super.initState();
    final double placeholderNumber = (widget.itemCount - 1) / 2;
    // 计算线框top值
    borderTop = placeholderNumber * widget.itemHeight;
    boxHeight = widget.itemHeight * widget.itemCount;

    //设置默认显示位置
    widget.defaultValue?.forEach((index, position) {
      tops[index] = -position * widget.itemHeight;
    });
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    theme = WeUI.getTheme(context);
  }

  onVerticalDragDown(DragDownDetails _, int index) {
    _startY = _.globalPosition.dy;
    _endY = tops[index] == null ? 0 : tops[index];
  }

  onVerticalDragUpdate(DragUpdateDetails _, int index) {
    this.setState(() {
      tops[index] = _endY + (_.globalPosition.dy - _startY);
    });
  }

  onVerticalDragEnd(int index) {
    final double itemHalfHeight = widget.itemHeight / 2;
    final double colHeight = widget.options[index].length * widget.itemHeight;
    final double currentTop = -tops[index];
    double top;

    // 判断边界值
    if (currentTop <= itemHalfHeight) {
      top = 0;
    } else if (currentTop >= colHeight - itemHalfHeight) {
      top = -(colHeight - widget.itemHeight);
    } else {
      int number = (tops[index] ~/ widget.itemHeight);
      number = number + (-tops[index] % widget.itemHeight >= itemHalfHeight ? -1 : 0);
      top = number * widget.itemHeight;
    }

    this.setState(() {
      tops[index] = top;
    });

    changeCallBack();
  }

  ///获取当前值
  List<WePickerItem> getResult() {
    List<WePickerItem> temp = [];
    widget.options.forEach((it) {
      int topsIndex = widget.options.indexOf(it);
      int position = tops.containsKey(topsIndex) ? (tops[topsIndex] ~/ widget.itemHeight).abs() : 0;
      WePickerItem item = it[position];
      temp.add(item);
    });
    return temp;
  }

  ///滚动回调
  void changeCallBack() {
    if (widget.onChange is PickerCallBack) {
      widget.onChange(getResult());
    }
  }

  DecoratedBox _renderMaskDecoratedBox(Alignment begin, Alignment end) {
    return DecoratedBox(
      decoration: BoxDecoration(
        gradient: new LinearGradient(begin: begin, end: end, colors: [
          Color.fromRGBO(255, 255, 255, 0.2),
          Color.fromRGBO(255, 255, 255, 1),
        ]),
      ),
    );
  }

  List<Widget> renderItem() {
    final List<Widget> columns = List.generate(widget.options.length, (index) {
      final column = widget.options[index];
      return Expanded(
        child: GestureDetector(
          onVerticalDragDown: (_) {
            onVerticalDragDown(_, index);
          },
          onVerticalDragUpdate: (_) {
            onVerticalDragUpdate(_, index);
          },
          onVerticalDragEnd: (_) {
            onVerticalDragEnd(index);
          },
          child: DecoratedBox(
            decoration: BoxDecoration(color: Colors.transparent),
            child: Stack(
              children: [
                Positioned(
                  top: tops[index] == null ? 0 : tops[index],
                  right: 0,
                  left: 0,
                  child: Padding(
                    padding: EdgeInsets.only(top: borderTop),
                    child: Column(
                      children: column.map((item) {
                        return SizedBox(
                          height: widget.itemHeight,
                          child: DefaultTextStyle(
                            style: TextStyle(fontSize: 16, color: Colors.black),
                            child: Align(alignment: Alignment.center, child: Text(item.label)),
                          ),
                        );
                      }).toList(),
                    ),
                  ),
                ),
                Positioned(
                  top: 0,
                  right: 0,
                  left: 0,
                  bottom: borderTop + widget.itemHeight,
                  child: _renderMaskDecoratedBox(Alignment.bottomCenter, Alignment.topCenter),
                ),
                Positioned(
                  top: borderTop + widget.itemHeight,
                  right: 0,
                  bottom: 0,
                  left: 0,
                  child: _renderMaskDecoratedBox(Alignment.topCenter, Alignment.bottomCenter),
                )
              ],
            ),
          ),
        ),
      );
    }).toList();

    return columns;
  }

  ///标题
  Widget renderTitleBar() {
    Widget cancel = Padding(
      padding: const EdgeInsets.only(left: 16.0, right: 16.0),
      child: DefaultTextStyle(
        style: TextStyle(fontSize: 14, color: Colors.blue),
        child: toTextWidget(widget.cancelButton, 'picker view cancel'),
      ),
    );

    Widget confirm = Padding(
      padding: const EdgeInsets.only(left: 16.0, right: 16.0),
      child: DefaultTextStyle(
        style: TextStyle(fontSize: 14, color: Colors.blue),
        child: toTextWidget(widget.confirmButton, 'picker view confirm'),
      ),
    );

    return SizedBox(
      height: 50,
      child: Row(
        crossAxisAlignment: CrossAxisAlignment.center,
        children: <Widget>[
          GestureDetector(
            onTap: widget.onCancel,
            child: cancel,
          ),
          Expanded(
            child: Align(
              alignment: Alignment.center,
              child: DefaultTextStyle(
                style: TextStyle(fontSize: 16, color: Colors.black87),
                child: toTextWidget(widget.title, 'picker view title'),
              ),
            ),
          ),
          GestureDetector(
            onTap: () => widget.onConfirm(getResult()),
            child: confirm,
          ),
        ],
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      color: Colors.white,
      height: boxHeight + 50,
      child: Column(
        children: <Widget>[
          renderTitleBar(),
          Divider(height: 1, color: theme.defaultBorderColor),
          Expanded(
            child: Stack(
              children: [
                Row(children: renderItem()),
                Positioned(
                  top: borderTop,
                  left: 0,
                  right: 0,
                  child: Divider(height: 1, color: theme.defaultBorderColor),
                ),
                Positioned(
                  top: borderTop + widget.itemHeight,
                  left: 0,
                  right: 0,
                  child: Divider(height: 1, color: theme.defaultBorderColor),
                )
              ],
            ),
          ),
        ],
      ),
    );
  }
}

class WePickerItem {
  final String label;
  final String value;

  WePickerItem({@required this.label, @required this.value});
}
